@@ -138,7 +138,7 @@ static gboolean setup_mbm(struct modem_info *modem)
 
 static gboolean setup_hso(struct modem_info *modem)
 {
-	const char *control = NULL, *application = NULL, *network = NULL;
+	const char *ctl = NULL, *app = NULL, *mdm = NULL, *net = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
@@ -150,23 +150,26 @@ static gboolean setup_hso(struct modem_info *modem)
 				info->number, info->label, info->sysattr);
 
 		if (g_strcmp0(info->sysattr, "Control") == 0)
-			control = info->devnode;
+			ctl = info->devnode;
 		else if (g_strcmp0(info->sysattr, "Application") == 0)
-			application = info->devnode;
+			app = info->devnode;
+		else if (g_strcmp0(info->sysattr, "Modem") == 0)
+			mdm = info->devnode;
 		else if (info->sysattr == NULL &&
 				g_str_has_prefix(info->devnode, "hso") == TRUE)
-			network = info->devnode;
+			net = info->devnode;
 	}
 
-	if (control == NULL || application == NULL)
+	if (ctl == NULL || app == NULL)
 		return FALSE;
 
-	DBG("control=%s application=%s network=%s",
-				control, application, network);
+	DBG("control=%s application=%s modem=%s network=%s",
+						ctl, app, mdm, net);
 
-	ofono_modem_set_string(modem->modem, "ControlPort", control);
-	ofono_modem_set_string(modem->modem, "ApplicationPort", application);
-	ofono_modem_set_string(modem->modem, "NetworkInterface", network);
+	ofono_modem_set_string(modem->modem, "Control", ctl);
+	ofono_modem_set_string(modem->modem, "Application", app);
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
 
 	return TRUE;
 }
@@ -206,7 +209,7 @@ static gboolean setup_gobi(struct modem_info *modem)
 
 static gboolean setup_sierra(struct modem_info *modem)
 {
-	const char *device = NULL;
+	const char *mdm = NULL, *app = NULL, *net = NULL, *diag = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
@@ -217,19 +220,63 @@ static gboolean setup_sierra(struct modem_info *modem)
 		DBG("%s %s %s %s", info->devnode, info->interface,
 						info->number, info->label);
 
-		if (g_strcmp0(info->interface, "255/255/255") == 0 &&
-					g_strcmp0(info->number, "03") == 0) {
-			device = info->devnode;
-			break;
+		if (g_strcmp0(info->interface, "255/255/255") == 0) {
+			if (g_strcmp0(info->number, "01") == 0)
+				diag = info->devnode;
+			if (g_strcmp0(info->number, "03") == 0)
+				mdm = info->devnode;
+			else if (g_strcmp0(info->number, "04") == 0)
+				app = info->devnode;
+			else if (g_strcmp0(info->number, "07") == 0)
+				net = info->devnode;
 		}
 	}
 
-	if (device == NULL)
+	if (mdm == NULL || app == NULL)
 		return FALSE;
 
-	DBG("device=%s", device);
+	DBG("modem=%s app=%s net=%s diag=%s", mdm, app, net, diag);
 
-	ofono_modem_set_string(modem->modem, "Device", device);
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "App", app);
+	ofono_modem_set_string(modem->modem, "Diag", diag);
+	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
+
+	return TRUE;
+}
+
+static gboolean setup_option(struct modem_info *modem)
+{
+	const char *aux = NULL, *mdm = NULL, *diag = NULL;
+	GSList *list;
+
+	DBG("%s", modem->syspath);
+
+	for (list = modem->devices; list; list = list->next) {
+		struct device_info *info = list->data;
+
+		DBG("%s %s %s %s", info->devnode, info->interface,
+						info->number, info->label);
+
+		if (g_strcmp0(info->interface, "255/255/255") == 0) {
+			if (g_strcmp0(info->number, "00") == 0)
+				mdm = info->devnode;
+			else if (g_strcmp0(info->number, "01") == 0)
+				diag = info->devnode;
+			else if (g_strcmp0(info->number, "02") == 0)
+				aux = info->devnode;
+		}
+
+	}
+
+	if (aux == NULL || mdm == NULL)
+		return FALSE;
+
+	DBG("aux=%s modem=%s diag=%s", aux, mdm, diag);
+
+	ofono_modem_set_string(modem->modem, "Aux", aux);
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "Diag", diag);
 
 	return TRUE;
 }
@@ -658,6 +705,7 @@ static struct {
 	{ "hso",	setup_hso,	"hsotype"		},
 	{ "gobi",	setup_gobi,	},
 	{ "sierra",	setup_sierra	},
+	{ "option",	setup_option	},
 	{ "huawei",	setup_huawei	},
 	{ "speedupcdma",setup_speedup	},
 	{ "speedup",	setup_speedup	},
@@ -851,6 +899,8 @@ static struct {
 	{ "hso",	"hso"				},
 	{ "gobi",	"qcserial"			},
 	{ "sierra",	"sierra"			},
+	{ "sierra",	"sierra_net"			},
+	{ "option",	"option",	"0af0"		},
 	{ "huawei",	"option",	"201e"		},
 	{ "huawei",	"cdc_ether",	"12d1"		},
 	{ "huawei",	"option",	"12d1"		},
